@@ -1,26 +1,21 @@
 /*
    GNU xhippo: a GTK-based playlist manager.
-   Copyright 1998, 1999, 2000, 2001, 2002, 2003 Adam Sampson <azz@gnu.org>
-
-   Please report bugs to bug-xhippo@gnu.org.
+   Copyright 1998, 1999, 2000, 2001, 2002, 2003, 2007 Adam Sampson <ats@offog.org>
 
    This file is part of GNU xhippo.
 
-   GNU xhippo is free software; you can redistribute and/or modify it
-   under the terms of that license as published by the Free Software
-   Foundation; either version 2 of the License, or (at your option)
-   any later version.
-   
-   GNU xhippo is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-   
-   You should have received a copy of the GNU General Public License
-   along with GNU xhippo; see the file COPYING. If not, write to the
-   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-   MA 02111-1307 USA, or see http://www.gnu.org/.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "xhippo.h"
@@ -29,69 +24,15 @@
 static void add_info_window_row(GtkWidget *table, gint pos,
 				const gchar *text1, const gchar *text2);
 static void popup_songinfo_window(int song);
-static void song_selected(gint row);
 
-/* Handle selections from the list. */
-#ifdef USEGTK2
-void handle_tree_select(GtkTreeSelection *selection, gpointer data) {
-  GtkTreeIter iter;
-  GtkTreeModel *model;
-  GtkTreePath *path;
-  gint row, *indices;
+/* Handle button presses on various widgets. */
+gboolean handle_button_press(GtkWidget *widget, GdkEventButton *event, 
+			     gpointer data) {
+  gint row = -1;
 
-  /* Give up if nothing was selected. */
-  if (!gtk_tree_selection_get_selected(selection, &model, &iter)) return;
-
-  path = gtk_tree_model_get_path(model, &iter);
-  indices = gtk_tree_path_get_indices(path);
-  row = indices[0];
-  gtk_tree_path_free(path);
-
-  song_selected(row);
-}
-#else
-void handle_list_select(GtkCList *gtklist, gint row, gint column,
-			GdkEventButton *event, gpointer data) {
-  song_selected(row);
-}
-#endif
-
-/* A song has been selected from the list. */
-static void song_selected(gint row) {
-  get_songinfo(row)->played = 1;
-  start_playing(row);
-  update_window();
-}
-
-/* Handle clicks on the status bar's event box. */
-gboolean handle_eventbox_click(GtkWidget *widget, GdkEventButton *event,
-			       gpointer data) {
-  if (event->button == 3) {
-    /* This is a right button click. We want the context menu, so pass
-       it on to handle_menu_popup. */
-    handle_menu_popup(widget, event, data);
-    return TRUE;
-  } else if (event->button == 1) {
-    /* This is a left button click. Pop up the song info window for
-       the current song. */
-    popup_songinfo_window(last_played);
-    return TRUE;
-  } else {
-    return FALSE;
-  }
-}
-
-/* Handle right clicks which bring up the popup menu. */
-gboolean handle_menu_popup(GtkWidget *widget, GdkEventButton *event, 
-			   gpointer data) {
-  gint state = 1;
- 
-  /* Return if this isn't an RMB click. */
-  if (event->button != 3) return FALSE;
-  
-  last_row_hit = -1;
   if (widget == filelist) {
-    /* The click was on the list. Get the selected file. */
+    /* The click was on the list. Get the clicked file. */
+
 #ifdef USEGTK2
     GtkTreePath *path;
 
@@ -99,35 +40,60 @@ gboolean handle_menu_popup(GtkWidget *widget, GdkEventButton *event,
 				      event->x, event->y,
 				      &path, NULL, NULL, NULL)) {
       gint *indices = gtk_tree_path_get_indices(path);
-      last_row_hit = indices[0];
+      row = indices[0];
       gtk_tree_path_free(path);
-    } else {
-      last_row_hit = -1;
     }
 #else
     gtk_clist_get_selection_info(GTK_CLIST(filelist),
 				 event->x, event->y,
-				 &last_row_hit, NULL);
+				 &row, NULL);
 #endif
-    
-  } else if (widget == eventbox) {
-    /* The click was on the status bar. If there's a song playing then
-       use that. */
-    if (playing) last_row_hit = last_played;
   }
 
-  /* If the click isn't on the list, disable the menu items. */
-  if (last_row_hit < 0) state = 0;
+  if (event->button == 3) {
+    /* Right-mouse-button click anywhere in the interface. */
+    gint state;
 
-  gtk_widget_set_sensitive(GTK_WIDGET(info_item), state);
-  gtk_widget_set_sensitive(GTK_WIDGET(up_item), state);
-  gtk_widget_set_sensitive(GTK_WIDGET(down_item), state);
-  gtk_widget_set_sensitive(GTK_WIDGET(delete_item), state);
+    /* Work out which row the user was interested in, and whether to
+       enable to menu items. */
+    if (widget == filelist && row >= 0) {
+      last_row_hit = row;
+      state = 1;
+    } else {
+      last_row_hit = -1;
+      state = 0;
+    }
 
-  gtk_menu_popup(GTK_MENU(popupmenu), NULL, NULL, NULL, NULL, 
-		 event->button, event->time);
+    gtk_widget_set_sensitive(GTK_WIDGET(info_item), state);
+    gtk_widget_set_sensitive(GTK_WIDGET(up_item), state);
+    gtk_widget_set_sensitive(GTK_WIDGET(down_item), state);
+    gtk_widget_set_sensitive(GTK_WIDGET(delete_item), state);
+    
+    gtk_menu_popup(GTK_MENU(popupmenu), NULL, NULL, NULL, NULL, 
+		   event->button, event->time);
 
-  return TRUE;
+    return TRUE;
+
+  } else if (event->button == 1 && widget == eventbox) {
+    /* Left click on the event box. */
+
+    popup_songinfo_window(last_played);
+
+    return TRUE;
+
+  } else if (event->button == 1 && widget == filelist && row != -1) {
+    /* Left click on a song. */
+
+    if (last_row_clicked != row) start_playing(row);
+    last_row_clicked = row;
+
+    return TRUE;
+
+  } else {
+    /* Another button click; use the normal handlers. */
+
+    return FALSE;
+  }
 }
 
 /* Handle the "Cancel" button in the file selector. */
@@ -198,14 +164,7 @@ void set_mini() {
 #endif
   } else {
     gtk_widget_show(list_box);
-
-    /* Fake the adjustment being changed, so that the scrollbar gets
-       placed in the right position when we come out of mini
-       mode. This is necessary because if the adjustment changes
-       (read: the list was scrolled) while the scrollbar is hidden,
-       the scrollbar will ignore the message it gets and thus will
-       come up in the wrong position when we unhide it. */
-    gtk_adjustment_value_changed(vadj);
+    sync_list();
 
 #ifdef USEGTK2
     gtk_window_resize(GTK_WINDOW(window), attribute_width, attribute_height);
@@ -494,7 +453,7 @@ void handle_dnd_drop(GtkWidget *widget, GdkDragContext *context,
        non-hostname mode), "file://myhost.org/home/azz/x" (from
        ROX-Filer in hostname mode), or "file:/home/azz/x" (from
        Konqueror). */
-    gchar *s = strdup(data->data), *p, *q;
+    gchar *s = strdup((const char *) data->data), *p, *q;
 
     p = s;
     while ((q = strchr(p, '\r'))) {
