@@ -1,26 +1,21 @@
 /*
    GNU xhippo: a GTK-based playlist manager.
-   Copyright 1998, 1999, 2000 Adam Sampson <azz@gnu.org>
-
-   Please report bugs to bug-xhippo@gnu.org.
+   Copyright 1998, 1999, 2000, 2007 Adam Sampson <ats@offog.org>
 
    This file is part of GNU xhippo.
 
-   GNU xhippo is free software; you can redistribute and/or modify it
-   under the terms of that license as published by the Free Software
-   Foundation; either version 2 of the License, or (at your option)
-   any later version.
-   
-   GNU xhippo is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-   
-   You should have received a copy of the GNU General Public License
-   along with GNU xhippo; see the file COPYING. If not, write to the
-   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-   MA 02111-1307 USA, or see http://www.gnu.org/.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "xhippo.h"
@@ -408,11 +403,11 @@ void read_window_state() {
 
 /* Handle a click on a preferences checkbox that represents a CF_BOOL. */
 static void prefs_bool_callback(GtkWidget *widget, gpointer data) {
-  gboolean *bool = (gboolean *) data;
+  gboolean *value = (gboolean *) data;
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
-    *bool = 1;
+    *value = 1;
   } else {
-    *bool = 0;
+    *value = 0;
   }
 }
 
