@@ -1,26 +1,21 @@
 /*
    GNU xhippo: a GTK-based playlist manager.
-   Copyright 1998, 1999, 2000 Adam Sampson <azz@gnu.org>
-
-   Please report bugs to bug-xhippo@gnu.org.
+   Copyright 1998, 1999, 2000, 2007 Adam Sampson <ats@offog.org>
 
    This file is part of GNU xhippo.
 
-   GNU xhippo is free software; you can redistribute and/or modify it
-   under the terms of that license as published by the Free Software
-   Foundation; either version 2 of the License, or (at your option)
-   any later version.
-   
-   GNU xhippo is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-   
-   You should have received a copy of the GNU General Public License
-   along with GNU xhippo; see the file COPYING. If not, write to the
-   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-   MA 02111-1307 USA, or see http://www.gnu.org/.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "xhippo.h"
@@ -28,9 +23,6 @@
 /* Prototypes for static functions. */
 static const char *find_swap_point(const char *a);
 void sort_as_appropriate(void);
-#ifdef HAVE_LIBID3
-static int get_id3_field(ID3Tag *tag, ID3_FrameID frameid, char *buf, int len);
-#endif
 #ifdef HAVE_LIBID3TAG
 static id3_latin1_t *get_id3tag_field(struct id3_tag *tag, char *id);
 #endif
@@ -56,12 +48,14 @@ guint contextid,
   playing = 0, 
   paused = 0;
 gint last_played = -1,
-  last_row_hit;
+  last_row_hit = -1,
+  last_row_clicked = -1;
 fileaction *last_action = NULL;
 
 #ifdef USEGTK2
 /* Get a GtkTreeIter pointing at a given row in the list. */
 static gboolean iter_from_position(gint item, GtkTreeIter *iter) {
+  if (item < 0) return FALSE;
   return gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(filestore), iter,
 				       NULL, item);
 }
@@ -83,19 +77,43 @@ static void thaw_list() {
 #endif
 }
 
-/* Select a given row in the list. It's assumed that this calls the
-   item-selected callback. */
+/* Select a given row in the list and scroll to make it visible if
+   necessary. */
 static void select_list_row(gint item) {
 #ifdef USEGTK2
   GtkTreeIter iter;
   GtkTreeSelection *selection;
 
-  iter_from_position(item, &iter);
+  if (!iter_from_position(item, &iter)) return;
   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(filelist));
   gtk_tree_selection_select_iter(selection, &iter);
 #else
   gtk_clist_select_row(GTK_CLIST(filelist), item, 0);
 #endif
+
+  if (mode_scroll_catchup && GTK_WIDGET_REALIZED(filelist)) {
+#ifdef USEGTK2
+    GdkRectangle row, vis;
+    GtkTreePath *path = gtk_tree_path_new();
+    gtk_tree_path_append_index(path, item);
+    
+    gtk_tree_view_get_cell_area(GTK_TREE_VIEW(filelist), path, NULL, &row);
+    gtk_tree_view_widget_to_tree_coords(GTK_TREE_VIEW(filelist),
+					row.x, row.y, &row.x, &row.y);
+    gtk_tree_view_get_visible_rect(GTK_TREE_VIEW(filelist), &vis);
+    
+    if ((row.y < vis.y)
+	|| (row.y + row.height) >= (vis.y + vis.height)) {
+      gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(filelist), path, NULL,
+				   TRUE, 0.5, 0.0);
+    }
+#else
+    if (gtk_clist_row_is_visible(GTK_CLIST(filelist), item)
+	!= GTK_VISIBILITY_FULL) {
+      gtk_clist_moveto(GTK_CLIST(filelist), item, 0, 0.5, 0.0);
+    }
+#endif
+  }
 }
 
 /* Clear the list. */
@@ -127,7 +145,7 @@ static void remove_list_row(gint item) {
   GtkTreeIter iter;
   gpointer sinfo;
 
-  iter_from_position(item, &iter);
+  if (!iter_from_position(item, &iter)) return;
   gtk_tree_model_get(GTK_TREE_MODEL(filestore), &iter,
 		     SONGINFO_COLUMN, &sinfo,
 		     -1);
@@ -166,6 +184,11 @@ static void kill_child(int signal) {
   kill(last_action->setgroup ? -childpid : childpid, signal);
 }
 
+/* Synchronise the list to what's currently playing. */
+void sync_list() {
+  select_list_row(last_played);
+}
+
 /* Stop playing. */
 void stop_playing() {
   if (!playing) return;
@@ -231,8 +254,6 @@ void start_playing(gint num) {
 
   if (childpid) stop_playing();
 
-  last_played = num;
-
   /* Get the extension of the filename. */
   name = get_songinfo(num)->name;
   last_action = get_action(name);
@@ -242,12 +263,19 @@ void start_playing(gint num) {
     return;
   }
   
+  last_played = num;
+  playing = 1;
+  paused = 0;
+  get_songinfo(num)->played = 1;
+
+  /* Update the interface. */
+  select_list_row(num);
+  update_window();
+
   /* Create a pipe to synchronise with the child. */
   if (pipe(pipefd) < 0) return;
 
   /* Fork off a child process to run the player in. */
-  playing = 1;
-  paused = 0;
   childpid = fork();
   if (childpid < 0) return;
 
@@ -339,8 +367,7 @@ void pick_prev() {
 	newitem = listcount - 1;
       }
     }
-    select_list_row(newitem);
-    /* ... which calls handle_list_select. */
+    start_playing(newitem);
   } else {
     status(_("No items to pick from!"));
   }
@@ -386,31 +413,12 @@ void pick_next() {
 	get_songinfo(newitem)->previous = last_played;
       }
     }
-    select_list_row(newitem);
-    /* ... which calls handle_list_select. */
+    start_playing(newitem);
   } else {
     status(_("No items to pick from!"));
   }
 }
 
-#ifdef HAVE_LIBID3
-/* Helper function to get a field as text from id3lib. 
-   Return 1 if successful, else return 0. */
-static int get_id3_field(ID3Tag *tag, ID3_FrameID frameid,
-			 char *buf, int len) {
-  ID3Frame *frame;
-  ID3Field *field;
-
-  frame = ID3Tag_FindFrameWithID(tag, frameid);
-  if (!frame) return 0;
-  field = ID3Frame_GetField(frame, ID3FN_TEXT);
-  if (!field) return 0;
-  ID3Field_GetASCII(field, buf, len);
-
-  return 1;
-}
-#endif
-
 #ifdef HAVE_LIBID3TAG
 static id3_latin1_t *get_id3tag_field(struct id3_tag *tag, char *id) {
   struct id3_frame const *frame;
@@ -433,30 +441,6 @@ static id3_latin1_t *get_id3tag_field(struct id3_tag *tag, char *id) {
 gchar *read_mp3_tag (const char *filename) {
   gchar *trackname = NULL;
 
-#ifdef HAVE_LIBID3
-  /* Use id3lib to read the tag. */
-  ID3Tag *tag;
-  char title[STRBUFSIZE], artist[STRBUFSIZE];
-
-  tag = ID3Tag_New();
-  if (!tag) return NULL;
-  if (ID3Tag_Link(tag, filename) == 0) goto out;
-
-  /* Get the title. */
-  if (!get_id3_field(tag, ID3FID_TITLE, title, STRBUFSIZE)) goto out;
-
-  /* Try several fields to get the artist. */
-  if (get_id3_field(tag, ID3FID_LEADARTIST, artist, STRBUFSIZE)
-      || get_id3_field(tag, ID3FID_BAND, artist, STRBUFSIZE)
-      || get_id3_field(tag, ID3FID_COMPOSER, artist, STRBUFSIZE)) {
-    trackname = malloc(strlen(artist) + 3 + strlen(title) + 1);
-    sprintf(trackname, "%s - %s", artist, title);
-  }
-
- out:
-  ID3Tag_Delete(tag);
-  return trackname;
-#else
 #ifdef HAVE_LIBID3TAG
   /* Use libid3tag to read the tag. */
   struct id3_file *f;
@@ -474,8 +458,9 @@ gchar *read_mp3_tag (const char *filename) {
   artist = get_id3tag_field(t, ID3_FRAME_ARTIST);
   if (!artist) goto out;
 
-  trackname = malloc(strlen(artist) + 3 + strlen(title) + 1);
-  sprintf(trackname, "%s - %s", artist, title);
+  trackname = malloc(strlen((const char *) artist) + 3
+                     + strlen((const char *) title) + 1);
+  sprintf(trackname, "%s - %s", (const char *) artist, (const char *) title);
 
  out: 
   if (title) free(title);
@@ -506,7 +491,6 @@ gchar *read_mp3_tag (const char *filename) {
   close(fd);
   return trackname;
 #endif /* HAVE_LIBID3TAG */
-#endif /* HAVE_LIBID3 */
 }
 
 /* Fill in entries in a songinfo structure that need a stat first. Use
@@ -873,7 +857,7 @@ songinfo *get_songinfo(gint number) {
   GtkTreeIter iter;
   gpointer sinfo;
 
-  iter_from_position(number, &iter);
+  if (!iter_from_position(number, &iter)) return NULL;
   gtk_tree_model_get(GTK_TREE_MODEL(filestore), &iter,
 		     SONGINFO_COLUMN, &sinfo,
 		     -1);
