@@ -1,26 +1,21 @@
 /*
    GNU xhippo: a GTK-based playlist manager.
-   Copyright 1998, 1999, 2000 Adam Sampson <azz@gnu.org>
-
-   Please report bugs to bug-xhippo@gnu.org.
+   Copyright 1998, 1999, 2000 Adam Sampson <ats@offog.org>
 
    This file is part of GNU xhippo.
 
-   GNU xhippo is free software; you can redistribute and/or modify it
-   under the terms of that license as published by the Free Software
-   Foundation; either version 2 of the License, or (at your option)
-   any later version.
-   
-   GNU xhippo is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-   
-   You should have received a copy of the GNU General Public License
-   along with GNU xhippo; see the file COPYING. If not, write to the
-   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-   MA 02111-1307 USA, or see http://www.gnu.org/.
+   This program is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "xhippo.h"
@@ -58,9 +53,8 @@ void status(const char *message) {
   gtk_statusbar_push(GTK_STATUSBAR(statusbar), contextid, message);
 }
 
-/* Update the status bar and scroll the window to the right place.
-   Called when we play a new song or we change in or out of mini
-   mode. */
+/* Update the status bar. Called when we play a new song or we change
+   in or out of mini mode. */
 void update_window() {
   gchar statbuf[STRBUFSIZE];
 
@@ -81,30 +75,6 @@ void update_window() {
 		 last_action->name);
       }
     }
-
-    if (mode_scroll_catchup && GTK_WIDGET_REALIZED(filelist)) {
-#ifdef USEGTK2
-      GdkRectangle row, vis;
-      GtkTreePath *path = gtk_tree_path_new();
-      gtk_tree_path_append_index(path, last_played);
-
-      gtk_tree_view_get_cell_area(GTK_TREE_VIEW(filelist), path, NULL, &row);
-      gtk_tree_view_widget_to_tree_coords(GTK_TREE_VIEW(filelist),
-					  row.x, row.y, &row.x, &row.y);
-      gtk_tree_view_get_visible_rect(GTK_TREE_VIEW(filelist), &vis);
-
-      if ((row.y < vis.y)
-	  || (row.y + row.height) >= (vis.y + vis.height)) {
-	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(filelist), path, NULL,
-				     TRUE, 0.5, 0.0);
-      }
-#else
-      if (gtk_clist_row_is_visible(GTK_CLIST(filelist), last_played)
-	  != GTK_VISIBILITY_FULL) {
-	gtk_clist_moveto(GTK_CLIST(filelist), last_played, 0, 0.5, 0.0);
-      }
-#endif
-    }
   } else {
     snprintf(statbuf, STRBUFSIZE, _("Stopped."));
   }
@@ -211,7 +181,7 @@ int main(int argc, char **argv) {
   /* Check that HOME is available. */
   if (!getenv("HOME")) {
     fprintf(stderr, _("You must set $HOME to your home directory.\n"));
-    exit(20);
+    return 20;
   }
 
   /* Get the CWD. */
@@ -299,7 +269,7 @@ int main(int argc, char **argv) {
   /* Create the event box for the status bar. */
   eventbox = gtk_event_box_new();
   gtk_signal_connect(GTK_OBJECT(eventbox), "button_press_event",
-		     GTK_SIGNAL_FUNC(handle_eventbox_click), NULL);
+		     GTK_SIGNAL_FUNC(handle_button_press), NULL);
   gtk_box_pack_start(GTK_BOX(all_box), eventbox, FALSE, FALSE, 0);
 
   /* Create the status bar. */
@@ -557,8 +527,7 @@ int main(int argc, char **argv) {
       break;
     case 'V':
       print_version();
-      exit(0);
-      break;
+      return 0;
     case 'h':
       print_version();
       g_print("\nUsage: xhippo [options] [playlist [playlist ...]]\n"
@@ -614,11 +583,9 @@ int main(int argc, char **argv) {
 "-h  --help         This help.\n"
 "\n"
 "Report bugs to bug-xhippo@gnu.org.\n");
-      exit(0);
-      break;
+      return 0;
     default:
-      exit(20);
-      break;
+      return 20;
     }
   }
 
@@ -632,16 +599,13 @@ int main(int argc, char **argv) {
   gtk_clist_set_column_width(GTK_CLIST(filelist), 0, 200);
   gtk_clist_set_shadow_type(GTK_CLIST(filelist), GTK_SHADOW_ETCHED_IN);
 
-  gtk_signal_connect(GTK_OBJECT(filelist), "select_row",
-		     GTK_SIGNAL_FUNC(handle_list_select), NULL);
   gtk_signal_connect(GTK_OBJECT(filelist), "button_press_event",
-  		     GTK_SIGNAL_FUNC(handle_menu_popup), NULL);
+  		     GTK_SIGNAL_FUNC(handle_button_press), NULL);
 #else
   {
     GtkTreeViewColumn *column;
     GtkCellRenderer *renderer;
     GtkTreeSelection *select;
-    GtkTreeIter iter;
 
     filestore = gtk_list_store_new(NUM_COLUMNS,
 				   G_TYPE_POINTER,
@@ -660,10 +624,8 @@ int main(int argc, char **argv) {
 
     select = gtk_tree_view_get_selection(GTK_TREE_VIEW(filelist));
     gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
-    g_signal_connect(G_OBJECT(select), "changed",
-		     G_CALLBACK(handle_tree_select), NULL);
     g_signal_connect(G_OBJECT(filelist), "button_press_event",
-		     G_CALLBACK(handle_menu_popup), NULL);
+		     G_CALLBACK(handle_button_press), NULL);
   }
 
   /* Since we add lots of items to the file list, we don't want it
@@ -731,6 +693,13 @@ int main(int argc, char **argv) {
   /* Show the everything box. */
   gtk_widget_show(all_box);
 
+  /* Attach the acceleration group to the window. */
+#ifdef USEGTK2
+  gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
+#else
+  gtk_accel_group_attach(accel_group, GTK_OBJECT(window));
+#endif
+
   /* Read the config file for the second time. */
   read_config_files(1);
 
@@ -741,28 +710,21 @@ int main(int argc, char **argv) {
   /* Read the window attributes as appropriate. */
   if (mode_save_window_pos) {
     read_window_state();
-  }
-  if (mode_start_mini) attribute_mini = 1;
 
-  /* Attach the acceleration group to the window. */
 #ifdef USEGTK2
-  gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
+    gtk_window_move(GTK_WINDOW(window), attribute_xpos, attribute_ypos);
+    if (attribute_width && attribute_height)
+      gtk_window_resize(GTK_WINDOW(window), attribute_width, attribute_height);
 #else
-  gtk_accel_group_attach(accel_group, GTK_OBJECT(window));
+    gtk_widget_set_uposition(window, attribute_xpos, attribute_ypos);
+    if (attribute_width && attribute_height)
+      gtk_widget_set_usize(window, attribute_width, attribute_height);
 #endif
+  }
 
   /* Set the window attributes and show it. */
-#ifdef USEGTK2
-  gtk_window_move(GTK_WINDOW(window), attribute_xpos, attribute_ypos);
-  if (attribute_width && attribute_height)
-    gtk_window_resize(GTK_WINDOW(window), attribute_width, attribute_height);
-#else
-  gtk_widget_set_uposition(window, attribute_xpos, attribute_ypos);
-  if (attribute_width && attribute_height)
-    gtk_widget_set_usize(window, attribute_width, attribute_height);
-#endif
+  if (mode_start_mini) attribute_mini = 1;
   if (attribute_mini) {
-    attribute_mini = 1;
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(minicheckbox), 1);
     set_mini();
   }
@@ -779,7 +741,5 @@ int main(int argc, char **argv) {
 
   /* Note that the interface doesn't exist any more, so you can't do
      anything interesting here. */
-
-  /* And exit cleanly. */
-  exit(0);
+  return 0;
 }
